#ifndef THREAD_H_
#define THREAD_H_

#include <list>
#include <set>
#include <string>
#include <pthread.h>
#include <stdio.h>

class Atom;
class Port;
class Scheduler;
class Component;
class Compound;

/***********************************
 * Class: Thread
 * Description:  Thread where multiple atoms are executed.
 ***********************************/
class Thread {

  public:
    typedef std::set<Atom*>  atom_set_type;
    typedef std::list<Atom*> atoms_type;
    typedef std::list<Port*> ports_type;

  protected:

    char*                  mName;  
    Scheduler*             mScheduler; 
    atoms_type             mAtoms;
    atom_set_type          mAtomSet;
    atoms_type             mNotifiedAtoms;
    atoms_type             mWaitingAtoms;
    ports_type             mElectedPorts;

    pthread_t              mThread;
    pthread_cond_t         mResume;
    pthread_mutex_t        mMutex;
   
  /*** Constructoris & Dectructors ***/
  public:
    Thread(const char* name, Scheduler* scheduler);
    virtual ~Thread();
 
  /*** Public Data-Access Methods ***/
  public:
    const char* name() { return mName; }
 
    inline pthread_cond_t* resume() { return &mResume; }
    inline pthread_mutex_t* mutex() { return &mMutex; }

   
  /*** Public Utility Methods ***/
  public:
            void              dump();
            static Component* findSubComponent( Compound* compound, const std::string& name);
            void              attach(Compound* fromComponent, const std::string& path);
            void              attachComponent(Component* component);
            void              attachCompound( Compound* compound);
            void              attachAtom(Atom *p); /*      */


  /*** Public Methods visible from appln. ***/
  public:
    void launch();
    void notify(Atom *atom);
   
  protected:
    void    run();
    Port*   wait();
        
  protected:
    static void* Run(void*);
    static int nThread;
};


#endif /* THREAD_H_*/
