/*
 *
 * Data structure for the state information 
 * of the system
 *
 */

#ifndef STATES_H
#define STATES_H

#include "connector.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

char* bip_strtok(char*& start, char token, int SIZE);

void setStatePlace(char *st, char* placeName, bool value);

void setVariableState(char *st, char* varName, bool value);
void setVariableState(char *st, char* varName, int value);
void setVariableState(char *st, char* varName, float value);
void setVariableState(char *st, char* varName, char* value);

bool  getBoolVariableState(char *&st);
int   getIntVariableState(char*& st);
float getFloatVariableState(char*& st);
char* getVariableState(char *&st);


class Leaf;
class Atomleaf;
class Connector;

class Node {
  protected:
    unsigned int index;
    bool         isleaf;
    int          nOutTrans;
  public:
    Node() { index = 0; nOutTrans = 0; }
    virtual ~Node() {}

    inline unsigned int getIndex() const          { return index; }
    inline void         setIndex (unsigned int i) { index = i; }
    inline bool         isLeaf()   const          { return isleaf;}
    inline int          outTrans() const          { return nOutTrans; }
    inline void         resetOutTrans()           { nOutTrans = 0; }
    inline void         incrOutTrans()            { nOutTrans++; }
    inline void         decrOutTrans()            { nOutTrans--; }

    bool        equals(Node* n);
    const char* printNode() const;


    void setStatePlace(char *st, char* placeName, bool value) {
      if (value) {
	 strcat(st, "T,");
      } else {
	 strcat(st, "F,");
      }
    }

    void setVariableState(char *st, char* varName, bool value) {
      if (value) {
	 strcat(st, ",T");
      } else {
	 strcat(st, ",F");
      }
    }
    void setVariableState(char *st, char* varName, int value) {
      char _numericData[20];
      sprintf(_numericData, ",%d", value);
      strcat(st, _numericData);
    }
    void setVariableState(char *st, char* varName, float value) {
      char _numericData[20];
      sprintf(_numericData, ",%f", value);
      strcat(st, _numericData);
    }
    void setVariableState(char *st, char* varName, char* value) {
      strcat(st, value);
    }

    char* getVariableState(char *&st) {
      return bip_strtok(st, ',', 10000);
    }

    char* string2bipState(char *st, char *&savePtr) {
      return (strtok_r(st, ",", &savePtr));
    }

    char* string2objState(char *&savePtr) {
      return (strtok_r(NULL, ",", &savePtr));
    }

    int getIntVariableState(char*& st) {
      char *_numericData;
      int value = 0;
      _numericData = bip_strtok(st, ',', 20);
      value = atoi(_numericData);
      free(_numericData);
      return value ;
    }

    int string2int(char *&savePtr) {
      char* token = strtok_r(NULL, ",", &savePtr);
      return (atoi(token));
    }

    float getFloatVariableState(char*& st) {
      char *_numericData;
      float value = 0;
      _numericData = bip_strtok(st, ',', 20);
      sscanf(_numericData, "%f", &value);
      free(_numericData);
      return value ;
    }

    float string2float(char *&savePtr) {
      char* token = strtok_r(NULL, ",", &savePtr);
      float value = 0;
      sscanf(token, "%f", &value);
      return value ;
    }

    bool getBoolVariableState(char *&st) {
      if (*st=='F') {
	st +=2 ;
	return false ;
      } else {
	st +=2 ;
	return true ;
      }
    }

    bool string2bool(char *&savePtr) {
      char* token = strtok_r(NULL, ",", &savePtr);
      if (!strcmp(token, "F")) {
        return false;
      }
      else {
        return true;
      }
    }

    bool string2bool(char *st, char *&savePtr) {
      char* token = strtok_r(st, ",", &savePtr);
      if (!strcmp(token, "F")) {
        return false;
      }
      else {
        return true;
      }
    }


    //    virtual int getNumChild() const = 0;
    //virtual Node* getChild(int i) const = 0;

    virtual int compare(Node*) const = 0;
    bool operator==(Node& c) { return this->compare(&c); }

    virtual unsigned hash(unsigned) const = 0;
    virtual Node* copy() = 0;

};

class Nonleaf : public Node {
  public:
    int numChild;
    Node** children;

    Nonleaf(int nChild);
    Nonleaf(const Nonleaf&);
    virtual ~Nonleaf();

    Node* getChild(int i) const;
    void  setChild(Node* child, int i);
/*     virtual const bool isLeaf() { return false; } */
    int getNumChild() const { return numChild; }

    int compare(Node*) const;
    unsigned hash(unsigned) const;
    Node* copy();

    //&&&
    int getNumLeaf();
    Leaf* getLeaf(int);
   //&&&     
};


class Leaf : public Node {
  private:
    //char* value;
    Atomleaf* aleaf;

  public:
    Leaf(Atomleaf* leaf);
    Leaf(const Leaf&);
    virtual ~Leaf();

    Atomleaf* getAleaf() const;
    void  setAleaf(Atomleaf*);
/*     virtual bool isLeaf() const { return true; } */
/*     int getNumChild() const { return 0; } */
/*     Node* getChild(int i) const { return NULL; } */

    int compare(Node*) const;
    unsigned hash(unsigned) const;
    Node* copy();
};

class Atomleaf {
  private:
    char* value;

  public:
    Atomleaf();
    Atomleaf(char*);
    Atomleaf(Atomleaf&);
    virtual ~Atomleaf();

    char* getVal();
    void setVal(char*);

    int compare(Atomleaf*) const;
    unsigned hash(unsigned) const;
    Atomleaf* copy();
};
//&&&

class TransNode {
  private:
    Node* m_state;
    Connector* m_trans;
    Interaction* m_inter;

  public:
    TransNode(Node* n, Connector* c) {
      m_state = n; 
      m_trans = c; 
      m_inter = c ? new Interaction(*(c->feasibleInter())) : NULL; 
    }
 
    ~TransNode() { 
      if (m_inter) { 
        delete m_inter; 
      }
    }

    Node*        state() { return m_state; }
    Connector*   trans() { return m_trans; }
    Interaction* inter() { return m_inter; }
};

#endif
