#ifndef COMPOUND_H_
#define COMPOUND_H_

#include <vector>

class Component;
class Connector;
class Priority;
class Clock;

/***********************************
 * Class: Compound
 * Description: Compound component
 ***********************************/
class Compound : public Component {
  public:

    typedef std::vector<Connector*> connectors_type;
    typedef std::vector<Component*> components_type;
    typedef std::vector<Priority*> priorities_type;

  /*** Protected data members ***/
  protected:
    components_type mComponents; /* subcomponents */
    connectors_type mConnectors; /* local connectors */
    priorities_type mPriorities; /* local priorities */

  /*** Constructoris & Dectructors ***/
  public:
  Compound(const char*, Compound*, const LogicalClock* = NULL);
  virtual ~Compound();

  /*** Public Data-Access Methods ***/
  public:
    connectors_type& connectors()
    {
      return mConnectors;
    }

    priorities_type&  priorities() 
    {
      return mPriorities;
    }
    components_type& components()
    {
    	return mComponents;
    }

  /*** Public Utility Methods ***/
  public:
    virtual bool       isAtom() const     { return false; }
    virtual bool       isCompound() const { return true; }
            void       activate(Scheduler*);
    virtual bool       globalState();                       
    virtual void       getBusyAtoms(Scheduler*);
            void       initPrioHeap(Scheduler*);
            void       setStaticPrioLevel();
    virtual void       setup(Scheduler*);
    virtual void       launch();
    virtual void dump(FILE* file) const;
            
  /*** Public Methods invoked from appln. ***/
  public:
    static  Compound* Create(int, char**, const LogicalClock* );
};


#endif /*COMPOUND_H_*/
