/*******************************************************************************
 *                            C++ Verimag
 *    
 *
 *------------------------------------------------------------------------------
 * 	            Distributed and Complex Systems - Gi�res
 *------------------------------------------------------------------------------
 * driver.h
 *------------------------------------------------------------------------------
 *  Driver for BIP2 Engine Implementation
 *  + class Driver        : generic driver implementation
 *------------------------------------------------------------------------------
 * Created by Jacques COMBAZ on Mon Nov 23 2009
 *
 ******************************************************************************/

#ifndef DRIVER_H
#define DRIVER_H
#include <list>
#include "port.h"
#include "component.h"
#include "structure.h"
#include "event.h"

class Handler;
class Driver;
class Port;
class Event;

class Driver {
  public:
		 
    typedef std::list<Port*> ports_type;
    typedef std::list<Event*> events_type;
  /*** Protected data members ***/
  protected:
   char*     Name;                        /* Name of the driver */
   ports_type   dEnvPorts;        /* list of envPorts of the driver */
   events_type  dEvents;          /* list of events of the driver */
   
  public:
   Handler*             dhandler;         /* the handler that manages the driver*/
  /*** Constructors & Dectructors ***/
  public:
    Driver(const char*);
    ~Driver();

  /*** Protected Data-Access Methods ***/
  public:
    inline char*                  name()   const  { return Name; }
    inline ports_type&     envPorts()  { return dEnvPorts; }   /* return the list of active envPorts  */
    inline events_type &    events()    { return dEvents; }     /* return the list of active envents   */
    inline void setHandler(Handler* handle)   { dhandler = handle; }  /* Pointing the handler*/ 
    inline Handler* handler()           { return dhandler; } 
  /*** Public Utility Methods ***/
  public:
           
            void map(Port*);            /* Adds a port to dEnvports and maps it to its event from dEvents */
            bool waiting();            /* return true iif the event handler is waiting for an event */
            void sync();
            
                
    virtual void initialize(ports_type&) = 0;
    virtual void launch()     = 0;
   
    virtual void sense()      = 0;
    virtual void actuate()    = 0;
    
    virtual void syncdata()   = 0;

};

#endif
