/*******************************************************************************
 *                            C++ Verimag
 *    
 *
 *------------------------------------------------------------------------------
 * 	            Distributed and Complex Systems - Gi�res
 *------------------------------------------------------------------------------
 * handler.h
 *------------------------------------------------------------------------------
 *  External (environment) event handler for BIP2 Engine implementation
 *  + class Handler       : external (environment) event handler
 *------------------------------------------------------------------------------
 * Created by Tesnim ABDELLATIF on Thu Mar 19 2009
 *
 ******************************************************************************/

#ifndef HANDLER_H
#define HANDLER_H

#include <string.h>
#include <stdio.h>
#include <list>
#include "port.h"
#include "structure.h"
#include "driver.h"


class Handler {
public:
	typedef std::list<Port*> ports_type;
    typedef std::list<Driver*> drivers_type;
  /*** Protected data members ***/
  protected:
    ports_type     mEnvPorts;        /* list of envPorts of the model */
    drivers_type   mDrivers;         /* list of active drivers */
    BitVector      mSynced;          /* synced drivers */

  public:
    pthread_cond_t* condition_event;
    pthread_mutex_t* mutex_event; 

  /*** Constructors & Dectructors ***/
  public:
    Handler();
    ~Handler();

  /*** Protected Data-Access Methods ***/
    inline void                   setEnvPorts(ports_type& ports)    { mEnvPorts = ports; }
    inline void                   setDrivers(drivers_type& drivers) { mDrivers = drivers; }
    
    inline void                   setCondition (pthread_cond_t* condition )  { condition_event = condition;}
    inline void                   setMutex     (pthread_mutex_t* mutex )  { mutex_event = mutex;}
  /*** Public Data-Access Methods ***/
  public:
    inline ports_type&      envPorts()  { return mEnvPorts; }   /* return the list of active envPorts  */
    inline drivers_type&    drivers()   { return mDrivers; }    /* return the list of active drivers   */

    inline BitVector               synced()    { return mSynced; }
    void   setDriver(Driver* driver);


  /*** Protected Utility Methods ***/
  protected:
            void           map(Port*, Driver*);                    /* link an envPort to a driver */
            void           map();                                  /* checking map envports to drivers */
            
            //bool           driversMap(Port* port);                 /* maps a port to its driver*/
            //void           setDrivers(Array<Port,false>& ports);

      
  /*** Public Utility Methods ***/
  public:
            void      initialize(ports_type&);         /* initialize the event handler w.r.t. envPorts of the model */
            void      launch();                               /* launch the event handler */
            bool      waiting();                               /* return true iif the event handler is waiting for an event */
            void      updateInputs();                         /* maps the ports to drivers*/

            void      notify();                               /* signaler la condition et relacher mutex */

            void      sense();                                /* Sync ports */
            void      actuate();                              /* actuate ports */
};

#endif
