/*
 *
 * Data structure for the state information 
 * of the system
 *
 */

#ifndef STATES_H
#define STATES_H

#include <cstring>
#include <cstdlib>

char* bip_strtok(char*& start, char token, int SIZE);


class Leaf;
class Atomleaf;
class Connector;

class Node {
  protected:
    unsigned int index;
    bool isleaf;
  public:
    Node() {index = 0;}
    virtual ~Node() {}

    bool isLeaf() const { return isleaf;}
    bool equals(Node* n);
    const char* printNode() const;

    unsigned int getIndex() const {return index;}
    void setIndex(unsigned int i) {index = i;}

    void setStatePlace(char *st, char* placeName, bool value) {
      if (value) {
	 strcat(st, "T,");
      } else {
	 strcat(st, "F,");
      }
    }

    void setVariableState(char *st, char* varName, bool value) {
      if (value) {
	 strcat(st, "T,");
      } else {
	 strcat(st, "F,");
      }
    }
    void setVariableState(char *st, char* varName, int value) {
      char _numericData[20];
      sprintf(_numericData, ",%d", value);
      strcat(st, _numericData);
    }
    void setVariableState(char *st, char* varName, char* value) {
      strcat(st, value);
    }

    char* getVariableState(char *&st) {
      return bip_strtok(st, ',', 200);
    }
    int getIntVariableState(char*& st) {
      char *_numericData;
      int value = 0;
      _numericData = bip_strtok(st, ',', 20);
      value = atoi(_numericData);
      free(_numericData);
      return value ;
    }
    bool getBoolVariableState(char *&st) {
      if (*st=='F') {
	st +=2 ;
	return false ;
      } else {
	st +=2 ;
	return true ;
      }
    }




    //    virtual int getNumChild() const = 0;
    //virtual Node* getChild(int i) const = 0;

    virtual int compare(Node*) const = 0;
    bool operator==(Node& c) { return this->compare(&c); }

    virtual unsigned hash(unsigned) const = 0;
    virtual Node* copy() = 0;

};

class Nonleaf : public Node {
  public:
    int numChild;
    Node** children;

    Nonleaf(int nChild);
    Nonleaf(const Nonleaf&);
    virtual ~Nonleaf();

    Node* getChild(int i) const;
    void  setChild(Node* child, int i);
/*     virtual const bool isLeaf() { return false; } */
    int getNumChild() const { return numChild; }

    int compare(Node*) const;
    unsigned hash(unsigned) const;
    Node* copy();

    //&&&
    int getNumLeaf();
    Leaf* getLeaf(int);
   //&&&     
};


class Leaf : public Node {
  private:
    //char* value;
    Atomleaf* aleaf;

  public:
    Leaf(Atomleaf* leaf);
    Leaf(const Leaf&);
    virtual ~Leaf();

    Atomleaf* getAleaf() const;
    void  setAleaf(Atomleaf*);
/*     virtual bool isLeaf() const { return true; } */
/*     int getNumChild() const { return 0; } */
/*     Node* getChild(int i) const { return NULL; } */

    int compare(Node*) const;
    unsigned hash(unsigned) const;
    Node* copy();
};

class Atomleaf {
  private:
    char* value;

  public:
    Atomleaf();
    Atomleaf(char*);
    Atomleaf(Atomleaf&);
    virtual ~Atomleaf();

    char* getVal();
    void setVal(char*);

    int compare(Atomleaf*) const;
    unsigned hash(unsigned) const;
    Atomleaf* copy();
};
//&&&

class TransNode {
  private:
    Node* m_state;
    Connector* m_trans;
    Interaction* m_inter;

  public:
    TransNode(Node* n, Connector* c) 
      { m_state = n; 
	m_trans = c; 
	m_inter = new Interaction(*(c->feasibleInter())); } 
    ~TransNode() { delete m_inter; }
    Node* state() { return m_state; }
    Connector* trans() { return m_trans; }
    Interaction* inter() { return m_inter; }
};

#endif
